@@ -57,15 +57,15 @@ diff_write_packet(const char* zone, uint32_t new_serial, uint16_t id,
 	uint32_t file_len = sizeof(uint32_t) + strlen(zone) +
 		sizeof(new_serial) + sizeof(id) + sizeof(seq_nr) + len;
 
-	df = fopen(filename, "a");
-	if(!df) {
-		log_msg(LOG_ERR, "could not open file %s for append: %s",
+	if (gettimeofday(&tv, NULL) != 0) {
+		log_msg(LOG_ERR, "could not set timestamp for %s: %s",
 			filename, strerror(errno));
 		return;
 	}
 
-	if (gettimeofday(&tv, NULL) != 0) {
-		log_msg(LOG_ERR, "could not set timestamp for %s: %s",
+	df = fopen(filename, "a");
+	if(!df) {
+		log_msg(LOG_ERR, "could not open file %s for append: %s",
 			filename, strerror(errno));
 		return;
 	}
@@ -98,15 +98,15 @@ diff_write_commit(const char* zone, uint32_t old_serial,
 	FILE *df;
 	uint32_t len;
 
-	df = fopen(filename, "a");
-	if(!df) {
-		log_msg(LOG_ERR, "could not open file %s for append: %s",
+	if (gettimeofday(&tv, NULL) != 0) {
+		log_msg(LOG_ERR, "could not set timestamp for %s: %s",
 			filename, strerror(errno));
 		return;
 	}
 
-	if (gettimeofday(&tv, NULL) != 0) {
-		log_msg(LOG_ERR, "could not set timestamp for %s: %s",
+	df = fopen(filename, "a");
+	if(!df) {
+		log_msg(LOG_ERR, "could not open file %s for append: %s",
 			filename, strerror(errno));
 		return;
 	}
@@ -136,7 +136,7 @@ diff_write_commit(const char* zone, uint32_t old_serial,
 }
 
 /*
- * <matthijs> Checksum to signal no data change occured (for example, by a
+ * Checksum to signal no data change occured (for example, by a
  * zonec run.
  */
 int
@@ -294,7 +294,6 @@ rrset_delete(namedb_type* db, domain_type* domain, rrset_type* rrset)
 	if(rrset->zone->ns_rrset == rrset) {
 		rrset->zone->ns_rrset = 0;
 	}
-#ifdef DNSSEC
 	if(domain == rrset->zone->apex && rrset_rrtype(rrset) == TYPE_RRSIG) {
 		for (i = 0; i < rrset->rr_count; ++i) {
 			if (rr_rrsig_type_covered(&rrset->rrs[i]) == TYPE_SOA) {
@@ -303,7 +302,6 @@ rrset_delete(namedb_type* db, domain_type* domain, rrset_type* rrset)
 			}
 		}
 	}
-#endif
 	/* recycle the memory space of the rrset */
 	for (i = 0; i < rrset->rr_count; ++i)
 		add_rdata_to_recyclebin(db, &rrset->rrs[i]);
@@ -398,7 +396,7 @@ delete_RR(namedb_type* db, const dname_type* dname,
 		ssize_t rdata_num;
 		int rrnum;
 		temptable = domain_table_create(temp_region);
-		/* <matthijs> this will ensure that the dnames in rdata are
+		/* This will ensure that the dnames in rdata are
 		 * normalized, conform RFC 4035, section 6.2
 		 */
 		rdata_num = rdata_wireformat_to_rdata_atoms(
@@ -469,7 +467,7 @@ add_RR(namedb_type* db, const dname_type* dname,
 		domain_add_rrset(domain, rrset);
 	}
 
-	/* <matthijs> dnames in rdata are normalized, conform RFC 4035,
+	/* dnames in rdata are normalized, conform RFC 4035,
 	 * Section 6.2
 	 */
 	rdata_num = rdata_wireformat_to_rdata_atoms(
@@ -543,7 +541,6 @@ add_RR(namedb_type* db, const dname_type* dname,
 		if(type == TYPE_NS) {
 			zone->ns_rrset = rrset;
 		}
-#ifdef DNSSEC
 		if(type == TYPE_RRSIG) {
 			int i;
 			for (i = 0; i < rrset->rr_count; ++i) {
@@ -553,7 +550,6 @@ add_RR(namedb_type* db, const dname_type* dname,
 				}
 			}
 		}
-#endif
 	}
 	return 1;
 }
@@ -1062,7 +1058,7 @@ mark_and_exit(nsd_options_t* opt, FILE* f, off_t commitpos, const char* desc)
 		fclose(f);
 	} else {
 		uint8_t c = 0;
-		fwrite(&c, sizeof(c), 1, f);
+		(void)write_data(f, &c, sizeof(c));
 		fclose(f);
 		log_msg(LOG_ERR, "marked xfr as failed: %s", desc);
 		log_msg(LOG_ERR, "marked xfr so that next reload can succeed");
@@ -1199,7 +1195,7 @@ store_ixfr_data(FILE *in, uint32_t len, struct diff_read_data* data, off_t* star
 		!diff_read_32(in, &new_serial) ||
 		!diff_read_16(in, &id) ||
 		!diff_read_32(in, &seq)) {
-		log_msg(LOG_INFO, "could not read ixfr store info: %s", strerror(errno));
+		log_msg(LOG_INFO, "could not read ixfr store info: file format error");
 		return 0;
 	}
 	len -= sizeof(uint32_t)*3 + sizeof(uint16_t) + strlen(zone_name);
@@ -1345,7 +1341,7 @@ diff_read_file(namedb_type* db, nsd_options_t* opt, struct diff_log** log,
 		}
 	}
 
-	/* <matthijs> always seek, to diff_pos or to beginning of the file. */
+	/* Always seek, to diff_pos or to beginning of the file. */
 	if (fseeko(df, 0, SEEK_SET)==-1) {
 		log_msg(LOG_INFO, "could not fseeko file %s: %s.", filename,
 				strerror(errno));
