@@ -140,10 +140,13 @@ typedef enum nsd_rc nsd_rc_type;
 #define TYPE_MAILA	254	/* A request for mail agent RRs (Obsolete - see MX) */
 #define TYPE_ANY	255	/* any type (wildcard) */
 
+#define TYPE_DLV	32769	/* RFC 4431 */
+#define PSEUDO_TYPE_DLV	RRTYPE_DESCRIPTORS_LENGTH
+
 #define MAXLABELLEN	63
 #define MAXDOMAINLEN	255
 
-#define MAXRDATALEN	64      /* This is more than enough, think multiple TXT.  */
+#define MAXRDATALEN	64      /* This is more than enough, think multiple TXT. */
 #define MAX_RDLENGTH	65535
 
 /* Maximum size of a single RR.  */
@@ -170,7 +173,7 @@ enum rdata_wireformat
 	RDATA_WF_BINARY, 	     /* Binary data (unknown length).  */
 	RDATA_WF_BINARYWITHLENGTH,   /* Binary data preceded by 1 byte length */
 	RDATA_WF_APL,		     /* APL data.  */
-	RDATA_WF_IPSECGATEWAY,	     /* IPSECKEY gateway ip4, ip6 or dname. */
+	RDATA_WF_IPSECGATEWAY	     /* IPSECKEY gateway ip4, ip6 or dname. */
 };
 typedef enum rdata_wireformat rdata_wireformat_type;
 
@@ -212,8 +215,8 @@ struct rrtype_descriptor
 	uint16_t    type;	/* RR type */
 	const char *name;	/* Textual name.  */
 	int         token;	/* Parser token.  */
-	uint8_t     minimum;	/* Minimum number of RDATAs.  */
-	uint8_t     maximum;	/* Maximum number of RDATAs.  */
+	uint32_t    minimum;	/* Minimum number of RDATAs.  */
+	uint32_t    maximum;	/* Maximum number of RDATAs.  */
 	uint8_t     wireformat[MAXRDATALEN]; /* rdata_wireformat_type */
 	uint8_t     zoneformat[MAXRDATALEN]; /* rdata_zoneformat_type  */
 };
