@@ -9,10 +9,7 @@
 
 #include <config.h>
 
-#if defined(TSIG) && defined(HAVE_SSL)
-
-#include <openssl/hmac.h>
-#include <openssl/sha.h>
+#if defined(HAVE_SSL)
 
 #include "tsig-openssl.h"
 #include "tsig.h"
@@ -66,12 +63,15 @@ tsig_openssl_init(region_type *region)
 	/* TODO: walk lookup supported algorithms table */
 	if (!tsig_openssl_init_algorithm(region, "md5", "hmac-md5","hmac-md5.sig-alg.reg.int."))
 		return 0;
+#ifdef HAVE_EVP_SHA1
 	if (!tsig_openssl_init_algorithm(region, "sha1", "hmac-sha1", "hmac-sha1."))
 		return 0;
-#ifdef SHA256_DIGEST_LENGTH
+#endif /* HAVE_EVP_SHA1 */
+
+#ifdef HAVE_EVP_SHA256
 	if (!tsig_openssl_init_algorithm(region, "sha256", "hmac-sha256", "hmac-sha256."))
 		return 0;
-#endif
+#endif /* HAVE_EVP_SHA256 */
 	return 1;
 }
 
@@ -118,4 +118,10 @@ final(void *context, uint8_t *digest, size_t *size)
 	*size = (size_t) len;
 }
 
-#endif /* defined(TSIG) && defined(HAVE_SSL) */
+void
+tsig_openssl_finalize()
+{
+	EVP_cleanup();
+}
+
+#endif /* defined(HAVE_SSL) */
