@@ -10,7 +10,7 @@
 
 #include <config.h>
 #include <stdlib.h>
-#include <openssl/sha.h>
+#include <ctype.h>
 
 #include "tsig.h"
 #include "tsig-openssl.h"
@@ -39,10 +39,13 @@ static size_t max_algo_digest_size = 0;
 
 tsig_lookup_algorithm_table tsig_supported_algorithms[] = {
 	{ TSIG_HMAC_MD5, "hmac-md5" },
+#ifdef HAVE_EVP_SHA1
 	{ TSIG_HMAC_SHA1, "hmac-sha1" },
-#ifdef SHA256_DIGEST_LENGTH
+#endif /* HAVE_EVP_SHA1 */
+
+#ifdef HAVE_EVP_SHA256
 	{ TSIG_HMAC_SHA256, "hmac-sha256" },
-#endif
+#endif /* HAVE_EVP_SHA256 */
         { 0, NULL }
 };
 
@@ -100,7 +103,7 @@ tsig_init(region_type *region)
 	tsig_key_table = NULL;
 	tsig_algorithm_table = NULL;
 
-#if defined(TSIG) && defined(HAVE_SSL)
+#if defined(HAVE_SSL)
 	return tsig_openssl_init(region);
 #endif
 	return 1;
@@ -129,6 +132,35 @@ tsig_add_algorithm(tsig_algorithm_type *algorithm)
 		max_algo_digest_size = algorithm->maximum_digest_size;
 }
 
+/**
+ * compare a tsig algorithm string lowercased
+ */
+int
+tsig_strlowercmp(const char* str1, const char* str2)
+{
+	while (str1 && str2 && *str1 != '\0' && *str2 != '\0') {
+		if(tolower((int)*str1) != tolower((int)*str2)) {
+			if(tolower((int)*str1) < tolower((int)*str2))
+				return -1;
+			return 1;
+		}
+		str1++;
+		str2++;
+	}
+	if (str1 && str2) {
+		if (*str1 == *str2)
+			return 0;
+		else if (*str1 == '\0')
+			return -1;
+	}
+	else if (!str1 && !str2)
+		return 0;
+	else if (!str1 && str2)
+		return -1;
+	return 1;
+}
+
+
 /*
  * Find an HMAC algorithm based on its short name.
  */
@@ -141,7 +173,7 @@ tsig_get_algorithm_by_name(const char *name)
 	     algorithm_entry;
 	     algorithm_entry = algorithm_entry->next)
 	{
-		if (strcmp(name, algorithm_entry->algorithm->short_name) == 0)
+		if (tsig_strlowercmp(name, algorithm_entry->algorithm->short_name) == 0)
 		{
 			return algorithm_entry->algorithm;
 		}
@@ -511,7 +543,7 @@ tsig_parse_rr(tsig_record_type *tsig, buffer_type *packet)
 	type = buffer_read_u16(packet);
 	klass = buffer_read_u16(packet);
 
-	/* <matthijs> TSIG not present */
+	/* TSIG not present */
 	if (type != TYPE_TSIG || klass != CLASS_ANY) {
 		buffer_set_position(packet, tsig->position);
 		return 1;
@@ -632,3 +664,11 @@ tsig_error_reply(tsig_record_type *tsig)
 		memset(tsig->mac_data, 0, tsig->mac_size);
 	tsig->mac_size = 0;
 }
+
+void
+tsig_finalize()
+{
+#if defined(HAVE_SSL)
+	tsig_openssl_finalize();
+#endif
+}
